@@ -37,6 +37,36 @@ def get_tmux_output(pl, *args):
 	return _run_tmux(lambda cmd: run_cmd(pl, cmd), args)
 
 
+def set_tmux_environment(varname, value, remove=True):
+	'''Set tmux global environment variable
+
+	:param str varname:
+		Name of the variable to set.
+	:param str value:
+		Variable value.
+	:param bool remove:
+		True if variable should be removed from the environment prior to 
+		attaching any client (runs ``tmux set-environment -r {varname}``).
+	'''
+	run_tmux_command('set-environment', '-g', varname, value)
+	if remove:
+		try:
+			run_tmux_command('set-environment', '-r', varname)
+		except subprocess.CalledProcessError:
+			# On tmux-2.0 this command may fail for whatever reason. Since it is 
+			# critical just ignore the failure.
+			pass
+
+
+def source_tmux_file(fname):
+	'''Source tmux configuration file
+
+	:param str fname:
+		Full path to the sourced file.
+	'''
+	run_tmux_command('source', fname)
+
+
 NON_DIGITS = re.compile('[^0-9]+')
 DIGITS = re.compile('[0-9]+')
 NON_LETTERS = re.compile('[^a-z]+')
