@@ -63,7 +63,7 @@ class TmuxRenderer(Renderer):
 		if segment_info:
 			r.update(segment_info)
 		if 'pane_id' in r:
-			varname = 'TMUX_PWD_' + r['pane_id'].lstrip('%')
+			varname = 'TMUX_PWD_' + str(r['pane_id'])
 			if varname in r['environ']:
 				r['getcwd'] = lambda: r['environ'][varname]
 		r['mode'] = mode
