@@ -18,7 +18,7 @@ def highlighted_string(s, group, **kwargs):
 	ret = {
 		'type': 'string',
 		'contents': s,
-		'highlight_group': [group],
+		'highlight_groups': [group],
 	}
 	ret.update(kwargs)
 	return ret
@@ -39,6 +39,14 @@ config = {
 				'theme': 'default',
 				'colorscheme': 'default',
 			},
+			'shell': {
+				'theme': 'default',
+				'colorscheme': 'default',
+			},
+			'wm': {
+				'theme': 'default',
+				'colorscheme': 'default',
+			},
 		},
 	},
 	'colors': {
@@ -63,20 +71,20 @@ config = {
 		'groups': {
 			'm1': 'g1',
 			'm2': 'g3',
-			'm3': {'fg': 'col11', 'bg': 'col12', 'attr': []},
+			'm3': {'fg': 'col11', 'bg': 'col12', 'attrs': []},
 		}
 	},
 	'colorschemes/default': {
 		'groups': {
-			'g1': {'fg': 'col5', 'bg': 'col6', 'attr': []},
-			'g2': {'fg': 'col7', 'bg': 'col8', 'attr': []},
-			'g3': {'fg': 'col9', 'bg': 'col10', 'attr': []},
+			'g1': {'fg': 'col5', 'bg': 'col6', 'attrs': []},
+			'g2': {'fg': 'col7', 'bg': 'col8', 'attrs': []},
+			'g3': {'fg': 'col9', 'bg': 'col10', 'attrs': []},
 		}
 	},
 	'colorschemes/test/default': {
 		'groups': {
-			'str1': {'fg': 'col1', 'bg': 'col2', 'attr': ['bold']},
-			'str2': {'fg': 'col3', 'bg': 'col4', 'attr': ['underline']},
+			'str1': {'fg': 'col1', 'bg': 'col2', 'attrs': ['bold']},
+			'str2': {'fg': 'col3', 'bg': 'col4', 'attrs': ['underline']},
 			'd1': 'g2',
 			'd2': 'm2',
 			'd3': 'm3',
@@ -84,7 +92,15 @@ config = {
 	},
 	'colorschemes/vim/default': {
 		'groups': {
-			'environment': {'fg': 'col3', 'bg': 'col4', 'attr': ['underline']},
+			'environment': {'fg': 'col3', 'bg': 'col4', 'attrs': ['underline']},
+		},
+	},
+	'colorschemes/wm/default': {
+		'groups': {
+			'hl1': {'fg': 'col1', 'bg': 'col2', 'attrs': ['underline']},
+			'hl2': {'fg': 'col2', 'bg': 'col1', 'attrs': []},
+			'hl3': {'fg': 'col3', 'bg': 'col1', 'attrs': ['underline']},
+			'hl4': {'fg': 'col2', 'bg': 'col4', 'attrs': []},
 		},
 	},
 	'themes/test/default': {
@@ -119,11 +135,10 @@ config = {
 		},
 	},
 	'themes/vim/default': {
-		'default_module': 'powerline.segments.common',
 		'segments': {
 			'left': [
 				{
-					'function': 'environment',
+					'function': 'powerline.segments.common.env.environment',
 					'args': {
 						'variable': 'TEST',
 					},
@@ -131,6 +146,27 @@ config = {
 			],
 		},
 	},
+	'themes/shell/default': {
+		'default_module': 'powerline.segments.common',
+		'segments': {
+			'left': [
+				highlighted_string('s', 'g1', width='auto'),
+			],
+		},
+	},
+	'themes/wm/default': {
+		'default_module': 'powerline.segments.common',
+		'segments': {
+			'left': [
+				highlighted_string('A', 'hl1'),
+				highlighted_string('B', 'hl2'),
+			],
+			'right': [
+				highlighted_string('C', 'hl3'),
+				highlighted_string('D', 'hl4'),
+			],
+		},
+	},
 }
 
 
@@ -639,30 +675,126 @@ class TestSegmentData(TestRender):
 		self.assertRenderEqual(p, '{56} 1S{56}>{56}3S{610}>>{910}3S{910}>{910}2S{10-}>>{--}')
 
 
+class TestShellEscapes(TestCase):
+	@with_new_config
+	def test_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1b[0;38;5;5;48;5;6m\xa0s\x1b[0;38;5;6;49;22m>>\x1b[0m')
+
+	@with_new_config
+	def test_tmux_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['additional_escapes'] = 'tmux'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1bPtmux;\x1b\x1b[0;38;5;5;48;5;6m\x1b\\\xa0s\x1bPtmux;\x1b\x1b[0;38;5;6;49;22m\x1b\\>>\x1bPtmux;\x1b\x1b[0m\x1b\\')
+
+	@with_new_config
+	def test_screen_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['additional_escapes'] = 'screen'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1bP\x1b\x1b[0;38;5;5;48;5;6m\x1b\\\xa0s\x1bP\x1b\x1b[0;38;5;6;49;22m\x1b\\>>\x1bP\x1b\x1b[0m\x1b\\')
+
+	@with_new_config
+	def test_fbterm_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_escape_style'] = 'fbterm'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1b[0m\x1b[1;5}\x1b[2;6}\xa0s\x1b[0m\x1b[1;6}\x1b[49m\x1b[22m>>\x1b[0m')
+
+	@with_new_config
+	def test_fbterm_tmux_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_escape_style'] = 'fbterm'
+		config['config']['common']['additional_escapes'] = 'tmux'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1bPtmux;\x1b\x1b[0m\x1b\x1b[1;5}\x1b\x1b[2;6}\x1b\\\xa0s\x1bPtmux;\x1b\x1b[0m\x1b\x1b[1;6}\x1b\x1b[49m\x1b\x1b[22m\x1b\\>>\x1bPtmux;\x1b\x1b[0m\x1b\\')
+
+	@with_new_config
+	def test_fbterm_screen_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_escape_style'] = 'fbterm'
+		config['config']['common']['additional_escapes'] = 'screen'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1bP\x1b\x1b[0m\x1b\x1b[1;5}\x1b\x1b[2;6}\x1b\\\xa0s\x1bP\x1b\x1b[0m\x1b\x1b[1;6}\x1b\x1b[49m\x1b\x1b[22m\x1b\\>>\x1bP\x1b\x1b[0m\x1b\\')
+
+	@with_new_config
+	def test_term_truecolor_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_truecolor'] = True
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1b[0;38;2;192;0;192;48;2;0;128;128m\xa0s\x1b[0;38;2;0;128;128;49;22m>>\x1b[0m')
+
+	@with_new_config
+	def test_term_truecolor_fbterm_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_escape_style'] = 'fbterm'
+		config['config']['common']['term_truecolor'] = True
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1b[0m\x1b[1;5}\x1b[2;6}\xa0s\x1b[0m\x1b[1;6}\x1b[49m\x1b[22m>>\x1b[0m')
+
+	@with_new_config
+	def test_term_truecolor_tmux_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_truecolor'] = True
+		config['config']['common']['additional_escapes'] = 'tmux'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1bPtmux;\x1b\x1b[0;38;2;192;0;192;48;2;0;128;128m\x1b\\\xa0s\x1bPtmux;\x1b\x1b[0;38;2;0;128;128;49;22m\x1b\\>>\x1bPtmux;\x1b\x1b[0m\x1b\\')
+
+	@with_new_config
+	def test_term_truecolor_screen_escapes(self, config):
+		from powerline.shell import ShellPowerline
+		import powerline as powerline_module
+		config['config']['common']['term_truecolor'] = True
+		config['config']['common']['additional_escapes'] = 'screen'
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, ShellPowerline, args=Args(config_path=[''])) as powerline:
+				self.assertEqual(powerline.render(segment_info={}, side='left'), '\x1bP\x1b\x1b[0;38;2;192;0;192;48;2;0;128;128m\x1b\\\xa0s\x1bP\x1b\x1b[0;38;2;0;128;128;49;22m\x1b\\>>\x1bP\x1b\x1b[0m\x1b\\')
+
+
 class TestVim(TestCase):
 	def test_environ_update(self):
 		# Regression test: test that segment obtains environment from vim, not 
 		# from os.environ.
-		from powerline.vim import VimPowerline
-		import powerline as powerline_module
-		import vim
-		vim.vars['powerline_config_paths'] = ['/']
-		with swap_attributes(config, powerline_module):
-			with vim_module._with('environ', TEST='abc'):
-				with get_powerline_raw(config, VimPowerline) as powerline:
-					window = vim_module.current.window
-					window_id = 1
-					winnr = window.number
-					self.assertEqual(powerline.render(window, window_id, winnr), b'%#Pl_3_8404992_4_192_underline#\xc2\xa0abc%#Pl_4_192_NONE_None_NONE#>>')
-					vim_module._environ['TEST'] = 'def'
-					self.assertEqual(powerline.render(window, window_id, winnr), b'%#Pl_3_8404992_4_192_underline#\xc2\xa0def%#Pl_4_192_NONE_None_NONE#>>')
+		import tests.vim as vim_module
+		with vim_module._with('globals', powerline_config_paths=['/']):
+			from powerline.vim import VimPowerline
+			import powerline as powerline_module
+			with swap_attributes(config, powerline_module):
+				with vim_module._with('environ', TEST='abc'):
+					with get_powerline_raw(config, VimPowerline) as powerline:
+						window = vim_module.current.window
+						window_id = 1
+						winnr = window.number
+						self.assertEqual(powerline.render(window, window_id, winnr), b'%#Pl_3_8404992_4_192_underline#\xc2\xa0abc%#Pl_4_192_NONE_None_NONE#>>')
+						vim_module._environ['TEST'] = 'def'
+						self.assertEqual(powerline.render(window, window_id, winnr), b'%#Pl_3_8404992_4_192_underline#\xc2\xa0def%#Pl_4_192_NONE_None_NONE#>>')
 
 	def test_local_themes(self):
 		# Regression test: VimPowerline.add_local_theme did not work properly.
 		from powerline.vim import VimPowerline
 		import powerline as powerline_module
 		with swap_attributes(config, powerline_module):
-			with get_powerline_raw(config, VimPowerline) as powerline:
+			with get_powerline_raw(config, VimPowerline, replace_gcp=True) as powerline:
 				powerline.add_local_theme('tests.matchers.always_true', {
 					'segment_data': {
 						'foo': {
@@ -674,7 +806,7 @@ class TestVim(TestCase):
 							{
 								'type': 'string',
 								'name': 'foo',
-								'highlight_group': ['g1']
+								'highlight_groups': ['g1']
 							}
 						]
 					}
@@ -693,6 +825,31 @@ class TestVim(TestCase):
 		sys.path.pop(0)
 
 
+class TestBar(TestRender):
+	def test_bar(self):
+		import powerline as powerline_module
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, powerline_module.Powerline, replace_gcp=True, ext='wm', renderer_module='bar') as powerline:
+				self.assertRenderEqual(
+					powerline,
+					'%{l}%{F#ffc00000}%{B#ff008000}%{+u} A%{F-B--u}%{F#ff008000}%{B#ffc00000}>>%{F-B--u}%{F#ff008000}%{B#ffc00000}B%{F-B--u}%{F#ffc00000}>>%{F-B--u}%{r}%{F#ffc00000}<<%{F-B--u}%{F#ff804000}%{B#ffc00000}%{+u}C%{F-B--u}%{F#ff0000c0}%{B#ffc00000}<<%{F-B--u}%{F#ff008000}%{B#ff0000c0}D %{F-B--u}'
+				)
+
+	@with_new_config
+	def test_bar_escape(self, config):
+		import powerline as powerline_module
+		config['themes/wm/default']['segments']['left'] = (
+			highlighted_string('%{asd}', 'hl1'),
+			highlighted_string('10% %', 'hl2'),
+		)
+		with swap_attributes(config, powerline_module):
+			with get_powerline_raw(config, powerline_module.Powerline, replace_gcp=True, ext='wm', renderer_module='bar') as powerline:
+				self.assertRenderEqual(
+					powerline,
+					'%{l}%{F#ffc00000}%{B#ff008000}%{+u} %%{asd}%{F-B--u}%{F#ff008000}%{B#ffc00000}>>%{F-B--u}%{F#ff008000}%{B#ffc00000}10%% %%%{F-B--u}%{F#ffc00000}>>%{F-B--u}%{r}%{F#ffc00000}<<%{F-B--u}%{F#ff804000}%{B#ffc00000}%{+u}C%{F-B--u}%{F#ff0000c0}%{B#ffc00000}<<%{F-B--u}%{F#ff008000}%{B#ff0000c0}D %{F-B--u}'
+				)
+
+
 if __name__ == '__main__':
 	from tests import main
 	main()
