@@ -191,9 +191,6 @@ def command(cmd):
 	elif cmd.startswith('hi '):
 		sp = cmd.split()
 		_highlights[sp[1]] = sp[2:]
-	elif cmd.startswith('function! Powerline_plugin_ctrlp'):
-		# Ignore CtrlP updating functions
-		pass
 	elif cmd.startswith('augroup'):
 		augroup = cmd.partition(' ')[2]
 		if augroup.upper() == 'END':
@@ -288,6 +285,12 @@ def eval(expr):
 		winnr = int(match.group(2))
 		varname = match.group(3)
 		return tabpages[tabnr].windows[winnr].vars[varname]
+	elif expr.startswith('type(function('):
+		import re
+		match = re.match(r'^type\(function\("([^"]+)"\)\) == 2$', expr)
+		if not match:
+			raise NotImplementedError(expr)
+		return 0
 	raise NotImplementedError(expr)
 
 
@@ -406,9 +409,11 @@ def _emul_bufnr(expr):
 
 
 @_vim
-def _emul_exists(varname):
-	if varname.startswith('g:'):
-		return varname[2:] in vars
+def _emul_exists(ident):
+	if ident.startswith('g:'):
+		return ident[2:] in vars
+	elif ident.startswith(':'):
+		return 0
 	raise NotImplementedError
 
 
@@ -850,9 +855,6 @@ class _WithBufName(object):
 		self.buffer = buffer
 		self.old = buffer.name
 		buffer.name = self.new
-		if buffer.name and os.path.basename(buffer.name) == 'ControlP':
-			buffer.vars['powerline_ctrlp_type'] = 'main'
-			buffer.vars['powerline_ctrlp_args'] = ['focus', 'byfname', '0', 'prev', 'item', 'next', 'marked']
 
 	def __exit__(self, *args):
 		self.buffer.name = self.old
